package com.zsc.edu.gateway.modules.iot.tsl.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zsc.edu.gateway.exception.ConstraintException;
import com.zsc.edu.gateway.modules.iot.tsl.dto.EventDto;
import com.zsc.edu.gateway.modules.iot.tsl.dto.ParamDto;
import com.zsc.edu.gateway.modules.iot.tsl.entity.Event;
import com.zsc.edu.gateway.modules.iot.tsl.entity.EventParam;
import com.zsc.edu.gateway.modules.iot.tsl.entity.Param;
import com.zsc.edu.gateway.modules.iot.tsl.mapper.EventMapper;
import com.zsc.edu.gateway.modules.iot.tsl.mapper.ParamMapper;
import com.zsc.edu.gateway.modules.iot.tsl.query.EventQuery;
import com.zsc.edu.gateway.modules.iot.tsl.repo.EventParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.repo.EventRepository;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.service.EventService;
import com.zsc.edu.gateway.modules.iot.tsl.service.ParamService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Yao
 */
@Service
@AllArgsConstructor
public class EventServiceImpl extends ServiceImpl<EventRepository, Event> implements EventService {
    private final EventMapper mapper;
    private final ParamMapper paramMapper;
    private final ParamRepository paramRepo;
    private final EventParamRepository eventParamRepo;
    private final ParamService paramService;

    /**
     * 新建物模型事件
     */
    @Override
    public Event create(EventDto dto) {
        if (baseMapper.findByName(dto.getName()) != null) {
            throw new ConstraintException("该事件已存在！");
        }
        Event event = mapper.toEntity(dto);
        save(event);
        eventParamRepo.insert(paramService.eventParamCreate(dto, event.getId()));
        return event;
    }
    /**
     * 更新物模型事件
     */
    @Override
    public Event update(EventDto dto, Long id) {
        Event event = baseMapper.selectById(id);
        mapper.convert(dto, event);
        updateById(event);
        paramService.eventParamUpdate(dto.getOutputs(), id);
        return event;
    }
    /**
     * 分页查询物模型事件
     *
     * @param query 查询表单
     * @param page  分页参数
     */
//    @Override
//    public IPage<Event> page(Page<Event> page, EventQuery query) {
//        return baseMapper.page(page, query);
//    }

    /**
     * 查询详情
     * @param id Id
     * @return Event
     */
    @Override
    public Event detail(Long id) {
        return baseMapper.selectById(id);
    }

    /**
     *删除
     */
    @Override
    public boolean delete(Long id) {
        removeById(id);
        LambdaQueryWrapper<EventParam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(EventParam::getEventId, id);
        List<EventParam> eventParams = eventParamRepo.selectList(wrapper);
        if (!eventParams.isEmpty()) {
            List<Long> paramIds = eventParams.stream()
                    .map(EventParam::getParamId)
                    .toList();
            paramRepo.deleteByIds(paramIds);
        }
        eventParamRepo.delete(wrapper);
        return true;
    }
}
