package com.zsc.edu.gateway.modules.iot.tsl.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zsc.edu.gateway.modules.iot.product.dto.ProductDto;
import com.zsc.edu.gateway.modules.iot.product.entity.ProductParam;
import com.zsc.edu.gateway.modules.iot.product.repo.ProductParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.dto.EventDto;
import com.zsc.edu.gateway.modules.iot.tsl.dto.ParamDto;
import com.zsc.edu.gateway.modules.iot.tsl.dto.ServeDto;
import com.zsc.edu.gateway.modules.iot.tsl.entity.EventParam;
import com.zsc.edu.gateway.modules.iot.tsl.entity.Param;
import com.zsc.edu.gateway.modules.iot.tsl.entity.ServeParam;
import com.zsc.edu.gateway.modules.iot.tsl.mapper.ParamMapper;
import com.zsc.edu.gateway.modules.iot.tsl.repo.EventParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ServeParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.service.ParamService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author zhuang
 */
@AllArgsConstructor
@Service
public class ParamServiceImpl extends ServiceImpl<ParamRepository, Param> implements ParamService {
    private final ParamMapper mapper;
    private final ParamMapper paramMapper;
    private final EventParamRepository eventParamRepository;
    private final ProductParamRepository productParamRepository;
    private final ServeParamRepository serveParamRepository;

    public List<Long> paramCreate(List<ParamDto> params) {
        List<Long> paramIds = new ArrayList<>();
        params.forEach(paramDto -> {
            Param param = mapper.toEntity(paramDto);
            baseMapper.insert(param);
            paramIds.add(param.getId());
        });
        return paramIds;
    }

    @Override
    public List<EventParam> eventParamCreate(EventDto dto, Long eventId) {
        return paramCreate(dto.getOutputs()).stream()
                .map(paramId -> new EventParam(eventId, paramId))
                .toList();
    }

    @Override
    public List<ProductParam> productParamCreate(ProductDto dto, Long productId) {
        return paramCreate(dto.getParams()).stream()
                .map(paramId -> new ProductParam(productId, paramId))
                .toList();
    }

    @Override
    public List<ServeParam> serveParamCreate(ServeDto dto, Long serveId) {
        List<ServeParam> serveParams = new ArrayList<>();
        serveParams.addAll(paramCreate(dto.getInputs()).stream()
                .map(paramId -> new ServeParam(serveId, paramId))
                .toList());
        serveParams.addAll(paramCreate(dto.getOutputs()).stream()
                .map(paramId -> new ServeParam(serveId, paramId))
                .toList());
        return serveParams;
    }

    public boolean paramUpdate(List<ParamDto> paramDto, List<Long> paramIds) {
        List<Param> params = baseMapper.selectByIds(paramIds);
        if (!params.isEmpty() && !paramDto.isEmpty()) {
            for (int i = 0; i < Math.min(params.size(), paramDto.size()); i++) {
                Param param = params.get(i);
                ParamDto dto = paramDto.get(i);
                paramMapper.convert(dto, param);
                baseMapper.updateById(param);
            }
        }
        return true;
    }

    @Override
    public boolean eventParamUpdate(List<ParamDto> paramDto, Long eventId) {
        return paramUpdate(paramDto, eventParamRepository.selectList(new LambdaQueryWrapper<EventParam>()
                        .eq(EventParam::getEventId, eventId))
                .stream()
                .map(EventParam::getParamId)
                .toList());
    }

    @Override
    public boolean productParamUpdate(List<ParamDto> paramDto, Long productId) {
        return paramUpdate(paramDto, productParamRepository.selectList(new LambdaQueryWrapper<ProductParam>()
                        .eq(ProductParam::getProductId, productId))
                .stream()
                .map(ProductParam::getParamId)
                .toList());
    }

    @Override
    public boolean serveParamUpdate(List<ParamDto> paramDto, Long serveId) {
        return paramUpdate(paramDto, serveParamRepository.selectList(new LambdaQueryWrapper<ServeParam>()
                        .eq(ServeParam::getServeId, serveId))
                .stream()
                .map(ServeParam::getParamId)
                .toList());
    }
}
