package com.zsc.edu.gateway.modules.iot.tsl.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zsc.edu.gateway.exception.ConstraintException;
import com.zsc.edu.gateway.modules.iot.tsl.dto.ParamDto;
import com.zsc.edu.gateway.modules.iot.tsl.dto.ServeDto;
import com.zsc.edu.gateway.modules.iot.tsl.entity.*;
import com.zsc.edu.gateway.modules.iot.tsl.mapper.ParamMapper;
import com.zsc.edu.gateway.modules.iot.tsl.mapper.ServeMapper;
import com.zsc.edu.gateway.modules.iot.tsl.query.ServeQuery;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ServeParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ServeRepository;
import com.zsc.edu.gateway.modules.iot.tsl.service.ParamService;
import com.zsc.edu.gateway.modules.iot.tsl.service.ServeService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;


/**
 * @author 15864
 */
@AllArgsConstructor
@Service
public class ServeServiceImpl extends ServiceImpl<ServeRepository, Serve> implements ServeService {

    private final ServeMapper mapper;
    private final ParamMapper paramMapper;
    private final ParamRepository paramRepository;
    private final ServeParamRepository serveParamRepository;
    private final ParamService paramService;
    /**
     * 新建功能
     */
    @Override
    public Serve create(ServeDto dto) {
        if (baseMapper.findByName(dto.getName()) != null) {
            throw new ConstraintException("该事件已存在！");
        }
        Serve serve = mapper.toEntity(dto);
        save(serve);
        serveParamRepository.insert(paramService.serveParamCreate(dto, serve.getId()));
        return serve;
    }
    /**
     * 更新功能
     */
    @Override
    public Serve update(ServeDto dto, Long id) {
        Serve serve = baseMapper.selectById(id);
        mapper.convert(dto, serve);
        updateById(serve);
        paramService.serveParamUpdate(dto.getInputs(), id);
        paramService.serveParamUpdate(dto.getOutputs(), id);
        return serve;
    }

    /**
     * 分页查询功能
     *
     * @param query 查询表单
     * @param page  分页参数
     */
//    @Override
//    public IPage<Serve> page(Page<Serve> page, ServeQuery query) {
//        return baseMapper.page(page, query);
//    }

    /**
     * 查询详情
     * @param id 主键
     * @return Serve
     */
    @Override
    public Serve detail(Long id) {
        return baseMapper.selectById(id);
    }
    @Override
    public Boolean delete(Long id) {
        removeById(id);
        LambdaQueryWrapper<ServeParam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ServeParam::getServeId, id);
        List<ServeParam> serveParam = serveParamRepository.selectList(wrapper);
        if (!serveParam.isEmpty()) {
            List<Long> paramIds = serveParam.stream()
                    .map(ServeParam::getParamId)
                    .toList();
            paramRepository.deleteByIds(paramIds);
        }
        serveParamRepository.delete(wrapper);
        return true;
    }
}
